from signalbot import Command, Context, MessageType

class MotDCommand(Command):
    def __init__(self, logger, cnc, bs):
        self.logger = logger
        self.cnc = cnc
        self.bs = bs

    async def handle(self, context: Context) -> None:
        if context.message.group == self.cnc:
            self.logger.info("social is ignoring cnc message")
            return

        if context.message.group == None:
            self.logger.info("social rebuffing DM message")
            reply = "I only reply to messages in the group chats"
            await context.send(reply)
            return

        if context.message.type == MessageType.DATA_MESSAGE:
            self.logger.info("social processing data message")
            reply = f'I heard {context.message.text}'
            self.logger.debug("social sending response")
            await context.send(reply)
            return

        if context.message.type == MessageType.GROUP_UPDATE_MESSAGE:
            self.logger.info("social processing group update")
            post_group = self.bot._groups_by_internal_id[context.message.group]
            prev_members = self.bs.get_members(context.message.group)
            new_member = False
            if prev_members:
                for member in post_group["members"]:
                    self.logger.debug(f'  looking for {member} in old group')
                    if member not in prev_members:
                        self.logger.debug("  found a new member of the group")
                        new_member = True
                for member in prev_members:
                    self.logger.debug(f'  looking for {member} in new group')
                    if member not in post_group["members"]:
                        self.logger.debug("  a member left the group")
            else:
                self.logger.debug("  found a new group")
                # TODO post TOC

            # update member cache
            self.bs.put_members(context.message.group, post_group["members"])
            self.bs.retain_only(list(self.bot._groups_by_internal_id.keys()))

            if new_member:
                motd = self.bs.get_motd(context.message.group)
                # TODO don't send too frequently
                if motd:
                    self.logger.info("sent the message of the day")
                    await context.send(motd)
                else:
                    self.logger.info("no message of the day to send")
            return



import re

from signalbot import Command, Context, MessageType

HELP_MESSAGE = """you can use these commands:
  list_groups: return known group names
  set_motd: set_motd group <newline> message"""


class CNCCommand(Command):
    def __init__(self, logger, managers, cnc, bs):
        self.logger = logger
        self.managers = managers
        self.cnc = cnc
        self.bs = bs

    def _get_group_info(self):
        my_group_ids = self.bs.list_groups()
        known_groups = self.bot._groups_by_internal_id
        group_ids = [ group_id for group_id in known_groups.keys() if group_id in my_group_ids]
        groups = [ known_groups[group_id] for group_id in group_ids ]
        group_info = [ { key: group[key] for key in ['name', 'internal_id'] } for group in groups ]
        group_info = sorted(group_info, key=lambda x: x['name'])
        for i, info in enumerate(group_info):
            info['tag'] = i
        return group_info
                             
    async def handle(self, context: Context) -> None:
        if context.message.group != self.cnc:  # guard against DMs
            self.logger.info("cnc ignoring DM message")
            reply = "I only reply to messages in the CNC channel\n"
            await context.send(reply)
            return

        if context.message.type == MessageType.DATA_MESSAGE:
            self.logger.info("cnc processing data message")
            if context.message.source_uuid not in self.managers:
                reply = "I only reply to messages from a manager\n"
                await context.send(reply)
                return
            ops = context.message.text.split(maxsplit=2)
            match(ops[0].lower()):
                case 'help':
                    self.logger.info("cnc sending help message")
                    await context.send(HELP_MESSAGE)
                    return

                case 'list_groups':
                    self.logger.info("cnc processing list request")
                    group_info = self._get_group_info()
                    reply = 'known groups:\n'
                    reply += '\n'.join([ f'{group['tag']}: {group["name"]}' for group in group_info ])
                    await context.send(reply)
                    return

                case 'set_motd':
                    self.logger.info("cnc processing set_mod request")
                    if len(ops) < 2:
                        reply = f'unrecognized set_motd syntax'
                        await context.send(reply)
                        return

                    group_info = self._get_group_info()                    
                    group_tag = ops[1]
                    motd = ops[2] if len(ops) == 3 else None

                    try: 
                        group_tag = int(group_tag)
                    except ValueError:
                        reply = f'invalid group index: {group_tag}'
                        await context.send(reply)
                        return

                    if group_tag > len(group_info):
                        reply = f'group index out of range: {group_tag}'
                        await context.send(reply)
                        return

                    group = group_info[group_tag]
                    self.bs.put_motd(group['internal_id'], motd)
                    if motd:
                        reply = f'motd set for group {group_tag} ({group['name']})'
                    else:
                        reply = f'motd cleared for group {group_tag} ({group['name']}'
                    await context.send(reply)
                    return

                case 'get_motd':
                    self.logger.info("cnc processing get_mod request")
                    if len(ops) < 2:
                        reply = f'unrecognized set_motd syntax'
                        await context.send(reply)
                        return
                    
                    group_info = self._get_group_info()                   
                    group_tag = ops[1]
                    motd = ops[2] if len(ops) == 3 else None

                    try: 
                        group_tag = int(group_tag)
                    except ValueError:
                        reply = f'invalid group index: {group_tag}'
                        await context.send(reply)
                        return

                    if group_tag >= len(group_info):
                        reply = f'group index out of range: {group_tag}'
                        await context.send(reply)
                        return
                    
                    group = group_info[group_tag]
                    motd = self.bs.get_motd(group['internal_id'])
                    if motd:
                        reply = f'motd for group {group_tag} ({group['name']}) is: \n{motd}'
                    else:
                        reply = f'there is no motd for group {group_tag} ({group['name']})'
                    await context.send(reply)
                    return


            reply = """unknown command, type "help" for a list"""
            await context.send(reply)
            return



from dotenv import load_dotenv
import logging
import os
import re
from signalbot import SignalBot, Config, SQLiteConfig, enable_console_logging

from . import cnc
from . import motd
from . import store

logger = logging.getLogger("welcomebot")

def loop():
    logger.info("main init")
    bot = SignalBot(
        Config(
            signal_service=os.environ["SIGNAL_SERVICE"],
            phone_number=os.environ["PHONE_NUMBER"],
            storage=SQLiteConfig(
                sqlite_db='signalbot_internal_state.db',
            )
        )
    )

    cnc_id = os.environ["WELCOME_CNC"]
    managers = re.split(r'[\s|,:]+', os.environ["WELCOME_MANAGER"])

    bot_store = store.BotStore(logger)
    bot.register(cnc.CNCCommand(logger, managers, cnc_id, bot_store), groups=[cnc_id]) # monitor other groups
    bot.register(motd.MotDCommand(logger, cnc_id, bot_store)) # monitor other groups
    logger.info("bot started")
    bot.start()


def main():
    # signalbot logs
    enable_console_logging(logging.WARNING)

    # welcomebot logs
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        "%(asctime)s %(name)s %(filename)s [%(levelname)s] - %(message)s"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    load_dotenv()

    loop()


if __name__ == "__main__":
    main()


import sqlite3

class BotStore():
    def __init__(self, logger, db="bot_memory.db"):
        self.logger = logger
        self.con = sqlite3.connect(db)
        cur = self.con.cursor()
        cur.execute("""   
             CREATE TABLE IF NOT EXISTS group_members (
                 group_id TEXT,
                 member_id TEXT
             );
        """)
        self.con.commit()
        cur = self.con.cursor()
        cur.execute("""   
             CREATE TABLE IF NOT EXISTS motd (
                 group_id TEXT,
                 motd TEXT
             );
        """)
        self.con.commit()
        cur.close()

    def __del__(self):
        self.con.close()

    def list_groups(self):
        cur = self.con.cursor()
        res = cur.execute('SELECT DISTINCT group_id FROM group_members')
        rows = res.fetchall()
        cur.close()
        return [ row[0] for row in rows ]

    def retain_only(self, known_groups):
        # TODO also prune old groups
        saved_groups = self.list_groups()
        obsolete_groups = [ group for group in saved_groups if group not in known_groups]
        if obsolete_groups:
            self.logger.debug(f'dropping {len(obsolete_groups)} obsolete groups')
            cur = self.con.cursor()
            placeholders = ', '.join('?' for _ in obsolete_groups)
            cur.executemany(f'DELETE FROM group_members WHERE group_id = ({placeholders})', obsolete_groups)
            self.con.commit()
            cur.close()
        else:
            self.logger.debug('no obsolete groups to prune')
        return obsolete_groups

    def get_members(self, group):
        cur = self.con.cursor()
        res = cur.execute(f'SELECT member_id FROM group_members WHERE group_id = "{group}"')
        rows = res.fetchall()
        cur.close()
        return [ row[0] for row in rows ]

    def put_members(self, group, members):
        cur = self.con.cursor()
        cur.execute(f'DELETE FROM group_members WHERE group_id = "{group}"')
        self.con.commit()
        rows = [ (group, member) for member in members ]
        cur = self.con.cursor()
        cur.executemany("INSERT INTO group_members VALUES(?, ?)", rows)
        self.con.commit()
        cur.close()

    def get_motd(self, group):
        cur = self.con.cursor()
        res = cur.execute(f'SELECT motd FROM motd WHERE group_id = "{group}"')
        row = res.fetchone()
        cur.close()
        return row[0] if row else None

    def put_motd(self, group, motd):
        cur = self.con.cursor()
        cur.execute(f'DELETE FROM motd WHERE group_id = "{group}"')
        self.con.commit()
        if motd:
            cur = self.con.cursor()
            cur.executemany("INSERT INTO motd VALUES(?, ?)", [ ( group, motd ) ] )
            self.con.commit()
            cur.close()
            return motd
        return None
